@@ -1,9 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-from typing import (
-    Optional,
-)
+from typing import Optional
 import pandas as pd
 from datetime import datetime, timedelta
 import requests
@@ -96,6 +94,12 @@ class forecast:
         self.lon = self.retrieve_hass_conf['lon']
         self.root = config_path
         self.logger = logger
+        self.var_load_cost = 'unit_load_cost'
+        self.var_prod_price = 'unit_prod_price'
+        self.forecast_dates = pd.date_range(start=self.start_forecast, 
+                                            end=self.end_forecast-self.freq, 
+                                            freq=self.freq).round(self.freq)
+        
         
     def get_weather_forecast(self, method: Optional[str] = 'scrapper') -> pd.DataFrame:
         """
@@ -194,6 +198,72 @@ class forecast:
         
         return P_PV_forecast
     
+    def get_forecast_days_csv(self, timedelta_days: Optional[int] = 1) -> pd.date_range:
+        """
+        Get the date range vector of forecast dates that will be used when \
+        loading a CSV file.
+        
+        :return: The forecast dates vector
+        :rtype: pd.date_range
+
+        """
+        start_forecast_csv = pd.Timestamp(datetime.now(), tz=self.time_zone).replace(hour=0, minute=0, second=0, microsecond=0)
+        end_forecast_csv = (start_forecast_csv + self.optim_conf['delta_forecast']).replace(microsecond=0)
+        forecast_dates_csv = pd.date_range(start=start_forecast_csv, 
+                                           end=end_forecast_csv+timedelta(days=timedelta_days)-self.freq, 
+                                           freq=self.freq).round(self.freq)
+        return forecast_dates_csv
+    
+    def get_forecast_out_from_csv(self, df_final: pd.DataFrame, 
+                                  forecast_dates_csv: pd.date_range,
+                                  csv_path: str) -> pd.DataFrame:
+        """
+        Get the forecast data as a DataFrame from a CSV file. The data contained \
+            in the CSV file should be a 24h forecast with the same frequency as \
+            the main 'freq' parameter in the configuration file. The timestamp \
+            will not be used and a new DateTimeIndex is generated to fit the \
+            timestamp index of the input data in 'df_final'.
+        
+        :param df_final: The DataFrame containing the input data.
+        :type df_final: pd.DataFrame
+        :param forecast_dates_csv: The forecast dates vector
+        :type forecast_dates_csv: pd.date_range
+        :param csv_path: The path to the CSV file
+        :type csv_path: str
+        :return: The data from the CSV file
+        :rtype: pd.DataFrame
+
+        """
+        days_list = pd.date_range(start=df_final.index[0], 
+                                  end=df_final.index[-1], 
+                                  freq='D')
+            
+        load_csv_file_path = self.root + csv_path
+        df_csv = pd.read_csv(load_csv_file_path, header=None, names=['ts', 'yhat'])
+        df_csv.index = forecast_dates_csv
+        df_csv.drop(['ts'], axis=1, inplace=True)
+        
+        forecast_out = pd.DataFrame()
+        for day in days_list:
+            first_elm_index = [i for i, x in enumerate(df_final.index.day == day.day) if x][0]
+            last_elm_index = [i for i, x in enumerate(df_final.index.day == day.day) if x][-1]
+            fcst_index = pd.date_range(start=df_final.index[first_elm_index],
+                                       end=df_final.index[last_elm_index], 
+                                       freq=df_final.index.freq)
+            first_hour = str(df_final.index[first_elm_index].hour)+":"+str(df_final.index[first_elm_index].minute)
+            last_hour = str(df_final.index[last_elm_index].hour)+":"+str(df_final.index[last_elm_index].minute)
+            if len(forecast_out) == 0:
+                forecast_out = pd.DataFrame(
+                    df_csv.between_time(first_hour, last_hour).values,
+                    index=fcst_index)
+            else:
+                forecast_tp = pd.DataFrame(
+                    df_csv.between_time(first_hour, last_hour).values,
+                    index=fcst_index)
+                forecast_out = pd.concat([forecast_out, forecast_tp], axis=0)
+        
+        return forecast_out
+    
     def get_load_forecast(self, days_min_load_forecast: Optional[int] = 3, method: Optional[str] = 'naive',
                           csv_path: Optional[str] = "/data/data_load_forecast.csv") -> pd.Series:
         """
@@ -203,7 +273,7 @@ class forecast:
             will be used to generate a naive forecast, defaults to 3
         :type days_min_load_forecast: int, optional
         :param method: The method to be used to generate load forecast, the options \
-            are 'csv' to load a CSV file of 'naive' for a persistance model, defaults to 'naive'
+            are 'csv' to load a CSV file or 'naive' for a persistance model, defaults to 'naive'
         :type method: str, optional
         :param csv_path: The path to the CSV file used when method = 'csv', \
             defaults to "/data/data_load_forecast.csv"
@@ -214,63 +284,117 @@ class forecast:
         """
         days_list = get_days_list(days_min_load_forecast)
         
-        forecast_dates = pd.date_range(start=self.start_forecast, 
-                                       end=self.end_forecast-self.freq, freq=self.freq).round(self.freq)
-        
-        self.logger.info("Retrieving data from hass for load forecast using method = "+method)
-        var_list = [self.var_load]
-        var_replace_zero = None
-        var_interp = [self.var_load]
-        time_zone_load_foreacast = None
-        
-        rh = retrieve_hass(self.retrieve_hass_conf['hass_url'], self.retrieve_hass_conf['long_lived_token'], 
-                           self.freq, time_zone_load_foreacast, self.root, self.logger)
-        
-        rh.get_data(days_list, var_list)
-        rh.prepare_data(self.retrieve_hass_conf['var_load'], load_negative = self.retrieve_hass_conf['load_negative'],
-                        set_zero_min = self.retrieve_hass_conf['set_zero_min'], 
-                        var_replace_zero = var_replace_zero, 
-                        var_interp = var_interp)
-        
-        df_load = rh.df_final.copy()
-        
-        df = df_load.copy()[[self.var_load_new]]
-        
-        if method == 'csv' or method == 'naive': # using a naive approach
-            mask_train = (df.index > days_list[:-2][0]) & (df.index <= days_list[:-2][-1])
-            df_train = df.copy().loc[mask_train]
-            df_train = df_train.rename(columns={self.var_load_new: 'y'})
+        if method == 'naive': # using a naive approach
+            self.logger.info("Retrieving data from hass for load forecast using method = "+method)
+            var_list = [self.var_load]
+            var_replace_zero = None
+            var_interp = [self.var_load]
+            time_zone_load_foreacast = None
             
-            mask_test = (df.index > days_list[:-2][-1]) & (df.index <= days_list[:-1][-1])
-            df_test = df.copy().loc[mask_test]
-            df_test = df_test.rename(columns={self.var_load_new: 'y'})
+            rh = retrieve_hass(self.retrieve_hass_conf['hass_url'], self.retrieve_hass_conf['long_lived_token'], 
+                               self.freq, time_zone_load_foreacast, self.root, self.logger)
             
-            mask_forecast_in_test = (df.index > days_list[:-3][-1]) & (df.index <= days_list[:-2][-1])
-            forecast_in_test = df.copy().loc[mask_forecast_in_test]
-            forecast_in_test = forecast_in_test.rename(columns={self.var_load_new: 'yhat'})
-            forecast_in_test.index = df_test.index
+            rh.get_data(days_list, var_list)
+            rh.prepare_data(self.retrieve_hass_conf['var_load'], load_negative = self.retrieve_hass_conf['load_negative'],
+                            set_zero_min = self.retrieve_hass_conf['set_zero_min'], 
+                            var_replace_zero = var_replace_zero, 
+                            var_interp = var_interp)
             
-            if method == 'csv': # reading from a csv file
-                start_forecast_csv = pd.Timestamp(datetime.now(), tz=self.time_zone).replace(hour=0, minute=0, second=0, microsecond=0)
-                end_forecast_csv = (start_forecast_csv + self.optim_conf['delta_forecast']).replace(microsecond=0)
-                forecast_dates_csv = pd.date_range(start=start_forecast_csv, 
-                                                   end=end_forecast_csv+timedelta(days=1)-self.freq, freq=self.freq).round(self.freq)
-                load_csv_file_path = self.root + csv_path
-                df_csv = pd.read_csv(load_csv_file_path, header=None, names=['ts', 'yhat'])
-                df_csv = pd.concat([df_csv, df_csv], axis=0)
-                df_csv.index = forecast_dates_csv
-                df_csv.drop(['ts'], axis=1, inplace=True)
-                forecast_out = df_csv.copy().loc[forecast_dates]
+            df = rh.df_final.copy()[[self.var_load_new]]
             
-            if method == 'naive': # using a naive approach
-                mask_forecast_out = (df.index > days_list[-1] - self.optim_conf['delta_forecast'])
-                forecast_out = df.copy().loc[mask_forecast_out]
-                forecast_out = forecast_out.rename(columns={self.var_load_new: 'yhat'})
-                # Force forecast_out length to avoid mismatches
-                forecast_out = forecast_out.iloc[0:len(forecast_dates)]
-                forecast_out.index = forecast_dates
+            mask_forecast_out = (df.index > days_list[-1] - self.optim_conf['delta_forecast'])
+            forecast_out = df.copy().loc[mask_forecast_out]
+            forecast_out = forecast_out.rename(columns={self.var_load_new: 'yhat'})
+            # Force forecast_out length to avoid mismatches
+            forecast_out = forecast_out.iloc[0:len(self.forecast_dates)]
+            forecast_out.index = self.forecast_dates
+        
+        elif method == 'csv': # reading from a csv file
+            forecast_dates_csv = self.get_forecast_days_csv()
+            load_csv_file_path = self.root + csv_path
+            df_csv = pd.read_csv(load_csv_file_path, header=None, names=['ts', 'yhat'])
+            df_csv = pd.concat([df_csv, df_csv], axis=0)
+            df_csv.index = forecast_dates_csv
+            df_csv.drop(['ts'], axis=1, inplace=True)
+            forecast_out = df_csv.copy().loc[self.forecast_dates]
             
         else:
             self.logger.error("Passed method is not valid")
 
-        return forecast_out['yhat']
\ No newline at end of file
+        return forecast_out['yhat']
+    
+    def get_load_cost_forecast(self, df_final: pd.DataFrame, method: Optional[str] = 'hp_hc_periods',
+                               csv_path: Optional[str] = "/data/data_load_cost_forecast.csv") -> pd.DataFrame:
+        """
+        Get the unit cost for the load consumption based on multiple tariff \
+        periods. This is the cost of the energy from the utility in a vector \
+        sampled at the fixed freq value.
+        
+        :param df_final: The DataFrame containing the input data.
+        :type df_final: pd.DataFrame
+        :param method: The method to be used to generate load cost forecast, \
+            the options are 'hp_hc_periods' for peak and non-peak hours contracts\
+            and 'csv' to load a CSV file, defaults to 'hp_hc_periods'
+        :type method: str, optional
+        :param csv_path: The path to the CSV file used when method = 'csv', \
+            defaults to "/data/data_load_cost_forecast.csv"
+        :type csv_path: str, optional
+        :return: The input DataFrame with one additionnal column appended containing
+            the load cost for each time observation.
+        :rtype: pd.DataFrame
+
+        """
+        if method == 'hp_hc_periods':
+            df_final[self.var_load_cost] = self.optim_conf['load_cost_hc']
+            list_df_hp = []
+            for key, period_hp in self.optim_conf['list_hp_periods'].items():
+                list_df_hp.append(df_final[self.var_load_cost].between_time(
+                    period_hp[0]['start'], period_hp[1]['end']))
+            for df_hp in list_df_hp:
+                df_final.loc[df_hp.index, self.var_load_cost] = self.optim_conf['load_cost_hp']
+        elif method == 'csv':
+            forecast_dates_csv = self.get_forecast_days_csv(timedelta_days=0)
+            forecast_out = self.get_forecast_out_from_csv(df_final,
+                                                          forecast_dates_csv,
+                                                          csv_path)
+            df_final[self.var_load_cost] = forecast_out
+        else:
+            self.logger.error("Passed method is not valid")
+            
+        return df_final
+    
+    def get_prod_price_forecast(self, df_final: pd.DataFrame, method: Optional[str] = 'constant',
+                               csv_path: Optional[str] = "/data/data_prod_price_forecast.csv") -> pd.DataFrame:
+        """
+        Get the unit power production price for the energy injected to the grid.\
+        This is the price of the energy injected to the utility in a vector \
+        sampled at the fixed freq value.
+        
+        :param df_input_data: The DataFrame containing all the input data retrieved
+            from hass
+        :type df_input_data: pd.DataFrame
+        :param method: The method to be used to generate the production price forecast, \
+            the options are 'constant' for a fixed constant value and 'csv'\
+            to load a CSV file, defaults to 'constant'
+        :type method: str, optional
+        :param csv_path: The path to the CSV file used when method = 'csv', \
+            defaults to "/data/data_load_cost_forecast.csv"
+        :type csv_path: str, optional
+        :return: The input DataFrame with one additionnal column appended containing
+            the power production price for each time observation.
+        :rtype: pd.DataFrame
+
+        """
+        if method == 'constant':
+            df_final[self.var_prod_price] = self.optim_conf['prod_sell_price']
+        elif method == 'csv':
+            forecast_dates_csv = self.get_forecast_days_csv(timedelta_days=0)
+            forecast_out = self.get_forecast_out_from_csv(df_final,
+                                                          forecast_dates_csv,
+                                                          csv_path)
+            df_final[self.var_prod_price] = forecast_out
+        else:
+            self.logger.error("Passed method is not valid")
+            
+        return df_final
+    
\ No newline at end of file
